# License: Public Domain

$KCODE = "u"
require "optparse"
require "set"

KEYS = "gkdtbpjcshnmrlaiueoyw".split(//)

CONSONANTS = [
  ["ㄱ", "g"],
  ["ㄲ", "kk"],
  ["ㅋ", "k"],
  ["ㄷ", "d"],
  ["ㄸ", "tt"],
  ["ㅌ", "t"],
  ["ㅂ", "b"],
  ["ㅃ", "pp"],
  ["ㅍ", "p"],
  ["ㅈ", "j"],
  ["ㅉ", "jj"],
  ["ㅊ", "c"],
  ["ㅅ", "s"],
  ["ㅆ", "ss"],
  ["ㅎ", "h"],
  ["ㄴ", "n"],
  ["ㅁ", "m"],
  ["ㅇ", "ng"],
  ["ㄹ", "r"],
  ["ㄹ", "r"],
]

def add_hangul(hangul, read)
  reads = [read]
  for pat, rep in
      [[/^c/, "ch"], [/^si/, "shi"], [/^r/, "l"], [/c$/, "ch"], [/weo/, "wo"], [/yi/, "ui"]]
    new_reads = []
    for read in reads
      new_reads.push(read)
      new_reads.push(read.gsub(pat, rep)) if read =~ pat
    end
    reads = new_reads
  end
  for read in reads
    @index[read] = hangul
    for i in 0...read.size
      @possible_next[read[0, i]] ||= Set.new()
      @possible_next[read[0, i]].add(read[i..i])
    end
  end
end

def get_key(read)
  (read.size - 1).downto(1) do |i|
    ph = @index[read[0, i]]
    return ph + (@prefix || "") + read[i..-1] if ph
  end
  return (@prefix || "") + read
end

@opts = OptionParser.getopts("", "prefix:")
@prefix = @opts["prefix"]
@index = {}
@possible_next = {}

for hangul, read in CONSONANTS
  add_hangul(hangul, read)
end

# euc_kr_hanguls.txt is extracted from:
# http://examples.oreilly.com/cjkvinfo/AppL/ksx1001.pdf
euc_hanguls = Set.new(File.readlines("euc_kr_hanguls.txt").map(){ |s| s.chomp() })
File.foreach("unicode_hanguls.txt") do |line|
  (hangul, read) = line.chomp().split(/\t/)
  if euc_hanguls.include?(hangul)
    add_hangul(hangul, read)
  end
end

for read in @index.keys.sort()
  key = get_key(read)
  if @prefix
    if @possible_next[read]
      puts([key, "", @index[read] + @prefix].join("\t"))
      for ch in KEYS
        if !@possible_next[read].include?(ch)
          puts([@index[read] + @prefix + ch, @index[read], (@index[ch] || ch) + @prefix].join("\t"))
        end
      end
    else
      puts([key, @index[read], @prefix].join("\t"))
    end
  else
    puts([key, "", @index[read]].join("\t"))
  end
end
